﻿' ----- Microsoft ADO.NET 4.0 Step by Step
'       Sample for Chapter 7 - Visual Basic Version
'       by Tim Patrick

Imports System.IO
Imports System.Text

Public Class Serialization
    Private SampleDataSet As DataSet

    Private Function BuildParentTable() As DataTable
        ' ----- Create a sample parent table.
        Dim result As DataTable
        Dim keyColumn As DataColumn

        ' ----- Design the table structure.
        result = New DataTable("Customer")
        keyColumn = result.Columns.Add("ID", GetType(Integer))
        result.Columns.Add("BusinessName", GetType(String))
        result.Columns.Add("AnnualFee", GetType(Decimal))
        result.Columns.Add("ContractDate", GetType(Date))
        result.PrimaryKey = {keyColumn}

        ' ----- Add some sample data.
        result.Rows.Add({1, "City Power & Light", 500@, #6/1/2008#})
        result.Rows.Add({2, "Lucerne Publishing", 300@, #1/1/2008#})
        result.Rows.Add({3, "Southridge Video", 350@, #2/15/2010#})

        Return result
    End Function

    Private Function BuildChildTable() As DataTable
        ' ----- Create a sample child table.
        Dim result As DataTable
        Dim keyColumn As DataColumn

        ' ----- Design the table structure.
        result = New DataTable("Order")
        keyColumn = result.Columns.Add("ID", GetType(Integer))
        keyColumn.AutoIncrement = True
        result.Columns.Add("CustomerID", GetType(Integer))
        result.Columns.Add("OrderDate", GetType(Date))
        result.Columns.Add("Subtotal", GetType(Decimal))
        result.Columns.Add("TaxRate", GetType(Decimal))
        result.Columns.Add("Total", GetType(Decimal),
            "Subtotal * (1 + TaxRate)")
        result.PrimaryKey = {keyColumn}

        ' ----- Add some sample data.
        result.Rows.Add({801, 1, #3/23/2010#, 342.94@, 0.0875@})
        result.Rows.Add({802, 3, #3/23/2010#, 100@, 0.0925@})
        result.Rows.Add({803, 3, #3/24/2010#, 59.99@, 0.0925@})
        result.Rows.Add({804, 1, #3/25/2010#, 342.94@, 0.0875@})
        result.Rows.Add({805, 2, #4/2/2010#, 180.08@, 0.0875@})

        Return result
    End Function

    Private Function BuildSampleDataSet() As DataSet
        ' ----- Build a sample data set of flight information.
        Dim result As DataSet
        Dim parentTable As DataTable
        Dim childTable As DataTable
        Dim tableLink As DataRelation

        ' ----- Add the two tables to the data set.
        result = New DataSet("SerializeSample")
        parentTable = BuildParentTable()
        childTable = BuildChildTable()
        result.Tables.Add(parentTable)
        result.Tables.Add(childTable)

        ' ----- Build the relationship between the tables.
        tableLink = New DataRelation("CustomerOrder", parentTable.Columns("ID"),
            childTable.Columns("CustomerID"), True)
        result.Relations.Add(tableLink)

        Return result
    End Function

    Private Function GetMappingColumn() As DataColumn
        ' ----- Locate the column from the SelectColumn field.
        Dim nameParts() As String
        Dim whichTable As DataTable

        ' ----- Identify the selected column.
        If (SelectColumn.SelectedIndex = -1) Then Return Nothing
        nameParts = Split(CStr(SelectColumn.SelectedItem), ".")
        If (nameParts(0) = "Parent") Then
            whichTable = SampleDataSet.Tables("Customer")
        Else
            whichTable = SampleDataSet.Tables("Order")
        End If
        Return whichTable.Columns(nameParts(1))
    End Function

    Private Sub Serialization_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' ----- Prepare the form.
        Dim scanColumn As DataColumn
        Dim parentTable As DataTable
        Dim childTable As DataTable

        ' ----- Prepare the core sample data.
        SampleDataSet = BuildSampleDataSet()
        parentTable = SampleDataSet.Tables("Customer")
        ParentContent.DataSource = parentTable
        childTable = SampleDataSet.Tables("Order")
        ChildContent.DataSource = childTable

        ' ----- Fill in the XML Write Mode options.
        OutputWriteMode.Items.Add(XmlWriteMode.WriteSchema)
        OutputWriteMode.Items.Add(XmlWriteMode.IgnoreSchema)
        OutputWriteMode.Items.Add(XmlWriteMode.DiffGram)
        OutputWriteMode.SelectedIndex = 0

        ' ----- Fill in the types of column mapping.
        ColumnMappingType.Items.Add(MappingType.Element)
        ColumnMappingType.Items.Add(MappingType.Attribute)
        ColumnMappingType.Items.Add(MappingType.SimpleContent)
        ColumnMappingType.Items.Add(MappingType.Hidden)

        ' ----- Fill in the list of columns.
        For Each scanColumn In parentTable.Columns
            SelectColumn.Items.Add("Parent." & scanColumn.ColumnName)
        Next scanColumn
        For Each scanColumn In childTable.Columns
            SelectColumn.Items.Add("Child." & scanColumn.ColumnName)
        Next scanColumn
        SelectColumn.SelectedIndex = 0
    End Sub

    Private Sub TextFields_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles _
            TableNamespace.Enter, TablePrefix.Enter
        ' ----- Highlight the entire text.
        CType(sender, TextBox).SelectAll()
    End Sub

    Private Sub ActGenerate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ActGenerate.Click
        ' ----- Generate the XML content using the settings.
        Dim holdBuffer As MemoryStream
        Dim charArray() As Char
        Dim decoder As UTF8Encoding

        ' ----- Set the XML namespace.
        SampleDataSet.Tables("Customer").Namespace = TableNamespace.Text.Trim
        SampleDataSet.Tables("Customer").Prefix = TablePrefix.Text.Trim
        SampleDataSet.Tables("Order").Namespace = TableNamespace.Text.Trim
        SampleDataSet.Tables("Order").Prefix = TablePrefix.Text.Trim

        ' ----- Indicate the relationship type.
        SampleDataSet.Relations(0).Nested = NestChildRecords.Checked

        ' ----- Clear any existing results.
        SerializedResults.Clear()

        ' ----- Build a memory stream to hold the results.
        holdBuffer = New MemoryStream(8192)
        SampleDataSet.WriteXml(holdBuffer, CType(OutputWriteMode.SelectedItem, XmlWriteMode))

        ' ----- Convert it to something displayable.
        decoder = New UTF8Encoding
        holdBuffer.Seek(0, SeekOrigin.Begin)
        charArray = New Char(decoder.GetCharCount(holdBuffer.ToArray) - 1) {}
        decoder.GetDecoder().GetChars(holdBuffer.ToArray, 0, charArray.Length, charArray, 0)
        SerializedResults.Text = New String(charArray)
    End Sub

    Private Sub SelectColumn_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SelectColumn.SelectedIndexChanged
        ' ----- Display the mapping for the selected column.
        Dim whichColumn As DataColumn

        ' ----- Identify the selected column.
        whichColumn = GetMappingColumn()
        If (whichColumn Is Nothing) Then Return

        ' ----- Set the option field to the current setting.
        ColumnMappingType.SelectedIndex = _
            ColumnMappingType.Items.IndexOf(whichColumn.ColumnMapping)
    End Sub

    Private Sub ColumnMappingType_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ColumnMappingType.SelectedIndexChanged
        ' ----- Display the mapping for the selected column.
        Dim whichColumn As DataColumn
        Dim newType As MappingType

        ' ----- Identify the selected column.
        whichColumn = GetMappingColumn()
        If (whichColumn Is Nothing) Then Return
        If (ColumnMappingType.SelectedIndex = -1) Then Return

        ' ----- Use the new setting to modify the column.
        newType = CType(ColumnMappingType.SelectedItem, MappingType)
        If (newType <> whichColumn.ColumnMapping) Then _
                whichColumn.ColumnMapping = newType
    End Sub
End Class
